#include <avr/io.h>
#include <avr/pgmspace.h>
#include <string.h>
#include <stdio.h>
#include "global.h"
#include "lcd.h"

u16 LCD_graphic_home,LCD_text_home;
u08 LCD_mem[LCD_MAX_SPALTEN*LCD_MAX_Y];
//u08 cx,cy;

void LCD_BusyCheck(void){
// while ((io(LCD_ADDR+(1<<LCD_CD))&3)!=3);
  u16 i;
  for(i=0;(i<0xffff);i++)
  {
  //asm("nop");
   if((io(LCD_ADDR+(1<<LCD_CD))&3)==3) return;
  }
}

u08 LCD_Status(void){
  LCD_BusyCheck();
  return io(LCD_ADDR+(1<<LCD_CD));
}

void LCD_Cmd0(u08 cmd){
  LCD_BusyCheck();
  io(LCD_ADDR+(1<<LCD_CD))=cmd;
}

void LCD_Cmd1(u08 cmd,u08 data1){
  LCD_BusyCheck();
  io(LCD_ADDR+(0<<LCD_CD))=data1;
  LCD_BusyCheck();
  io(LCD_ADDR+(1<<LCD_CD))=cmd;
}

void LCD_Cmd2(u08 cmd,u08 data1,u08 data2){
  LCD_BusyCheck();
  io(LCD_ADDR+(0<<LCD_CD))=data1;
  LCD_BusyCheck();
  io(LCD_ADDR+(0<<LCD_CD))=data2;
  LCD_BusyCheck();
  io(LCD_ADDR+(1<<LCD_CD))=cmd;
}

u08 LCD_Read(void){
  LCD_Cmd0(0xC5);
  LCD_BusyCheck();
  return io(LCD_ADDR+(0<<LCD_CD));
}

u08 LCD_ReadInc(void){
  LCD_Cmd0(0xC1);
  LCD_BusyCheck();
  return io(LCD_ADDR+(0<<LCD_CD));
}

u08 LCD_ReadDec(void){
  LCD_Cmd0(0xC3);
  LCD_BusyCheck();
  return io(LCD_ADDR+(0<<LCD_CD));
}

inline void LCD_SetPtr(u16 address){
  LCD_Cmd2(0x24,address & 0x00ff, address >> 8);
}

void LCD_SetTextScreen(u08 screen){
	if(screen<LCD_MAX_TEXT_SCREENS)
	{
		LCD_SetTextPtr(LCD_TEXT_HOME+(u16)screen*(LCD_MAX_ZEILEN*LCD_MAX_SPALTEN));
	}
}

void LCD_ShowTextScreen(u08 screen){
	if(screen<LCD_MAX_TEXT_SCREENS)
	{
		LCD_SetTextHome(LCD_TEXT_HOME+(u16)screen*(LCD_MAX_ZEILEN*LCD_MAX_SPALTEN));
	}
}


void LCD_SetTextPtr(u16 text_home_addr){
  LCD_SetPtr(text_home_addr);
  LCD_text_home = text_home_addr;
}

void LCD_SetTextHome(u16 text_home_addr){
  LCD_Cmd2(0x40,text_home_addr & 0x00ff, text_home_addr >> 8);
  LCD_text_home = text_home_addr;
}

void LCD_SetTextArea(u16 text_area){
  LCD_Cmd2(0x41,text_area & 0x00ff, text_area >> 8);
}

void LCD_SetGraphicHome(u16 graphic_home_addr){
  LCD_Cmd2(0x42,graphic_home_addr & 0x00ff, graphic_home_addr >> 8);
  LCD_graphic_home = graphic_home_addr;
}

inline void LCD_SetGraphicArea(u16 graphic_area){
  LCD_Cmd2(0x43,graphic_area & 0x00ff, graphic_area >> 8);
}

void LCD_SetMergeMode(u08 mode){
  LCD_Cmd0(mode);
}

inline void LCD_SetDisplayMode(u08 mode){
  LCD_Cmd0(mode);
}

inline void LCD_SetCursorPos(u08 x,u08 y){
  LCD_Cmd2(0x21,x,y);
//  cx=x;
//  cy=y;
}

inline void LCD_SetCursorHeight(u08 lines){
  LCD_Cmd0(0xa0 | ((lines-1) & 7));
}

inline void LCD_WriteInc(u08 data){
  LCD_Cmd1(0xc0, data);
}

inline void LCD_WriteDec(u08 data){
  LCD_Cmd1(0xc2, data);
}

inline void LCD_Write(u08 data){
  LCD_Cmd1(0xc4, data);
}

inline void LCD_AutoWriteOn(void){
  LCD_Cmd0(0xb0);
}

void LCD_AutoWrite(u08 data){
  while ( !((io(LCD_ADDR+(1<<LCD_CD)))&8) );
  io(LCD_ADDR+(0<<LCD_CD))=data;
}

inline void LCD_AutoWriteOff(void){
  LCD_Cmd0(0xb2);
}

void LCD_SetPixel(u08 x,u08 y){
  if((x>LCD_MAX_X)||(y>LCD_MAX_Y)) return;
  LCD_SetPtr(LCD_graphic_home+y*40+x/6);
  LCD_Cmd0(0xF8|(5-x%6));
}

void LCD_ClearPixel(u08 x,u08 y){
  if((x>LCD_MAX_X)||(y>LCD_MAX_Y)) return;
  LCD_SetPtr(LCD_graphic_home+y*40+x/6);
  LCD_Cmd0(0xF0|(5-x%6));
}

void LCD_SetMemPixel(u08 x,u08 y){
  setbit(LCD_mem[y*40+x/6],(5-x%6));
}

void LCD_ClearMemPixel(u08 x,u08 y){
  clearbit(LCD_mem[y*40+x/6],(5-x%6));
}

void LCD_TransferMemPage(void){
  u16 i;
  LCD_SetPtr(LCD_graphic_home);
  LCD_AutoWriteOn();
  for(i=0;i<LCD_MAX_SPALTEN*LCD_MAX_Y;i++)
    LCD_AutoWrite(LCD_mem[i]);
  LCD_AutoWriteOff();
  LCD_SetPtr(LCD_graphic_home);
}

void LCD_ClrMemPage(void){
  memset(LCD_mem,0,LCD_MAX_SPALTEN*LCD_MAX_Y);
}

void LCD_TextClr(void){
  u16 i;
  LCD_SetPtr(LCD_TEXT_HOME);
  LCD_AutoWriteOn();
  for(i=0;i<LCD_MAX_SPALTEN*LCD_TEXT_ZEILEN;i++)
    LCD_AutoWrite(0);
  LCD_AutoWriteOff();
  LCD_SetPtr(LCD_text_home);
}

void LCD_GraphicClr(void){
  u16 i;
  LCD_SetPtr(LCD_graphic_home);
  LCD_AutoWriteOn();
  for(i=0;i<LCD_MAX_SPALTEN*LCD_MAX_Y;i++)
    LCD_AutoWrite(0);
  LCD_AutoWriteOff();
  LCD_SetPtr(LCD_graphic_home);
}

void LCD_Init(void){
//  LCD_BusyCheck();
  LCD_SetGraphicHome(LCD_GRAPHIC_HOME);
  LCD_SetGraphicArea(LCD_MAX_SPALTEN);
  LCD_SetTextHome(LCD_TEXT_HOME);
  LCD_SetTextArea(LCD_MAX_SPALTEN);
  LCD_TextClr();
  LCD_SetGraphicHome(LCD_GRAPHIC_HOME);
  LCD_GraphicClr();
  LCD_SetMergeMode(0x81);
  LCD_SetDisplayMode(0x9F);
  LCD_SetPtr(LCD_TEXT_HOME);
  LCD_SetCursorHeight(8);
  LCD_SetCursorPos(0,0);
  LCD_ClrMemPage();
  LCD_TransferMemPage();
}


void LCD_PrintChar(u08 c){
  if((c<32)||(c>159)) return;
  LCD_Cmd1(0xC0,c-0x20);
}
/*
void LCD_SetGraphicPage(u08 page){
  if(page) LCD_graphic_home = LCD_GRAPHIC_HOME + LCD_MAX_SPALTEN*LCD_MAX_Y;
  else LCD_graphic_home = LCD_GRAPHIC_HOME;
}

void LCD_ShowGraphicPage(u08 page){
  if (page) LCD_SetGraphicHome(LCD_GRAPHIC_HOME + LCD_MAX_SPALTEN*LCD_MAX_Y);
  else LCD_SetGraphicHome(LCD_GRAPHIC_HOME);
}
*/
void LCD_GotoXY(u08 x,u08 y){
  LCD_SetPtr(LCD_text_home+x+y*LCD_MAX_SPALTEN);
//  LCD_SetCursorPos(x,y);
}

void LCD_PrintString(u08 *s){
  u08 i;
  for(i=0;s[i]!=0;i++)
  {
    LCD_PrintChar(s[i]);
  }
}

void LCD_PrintProgString(u08* s){
  for(;PRG_RDB(s)!=0;LCD_PrintChar(PRG_RDB(s++)));
    
}

void clr_text(void)
{
 u16 i;
 LCD_GotoXY(0,1);
 for(i=0;i<(LCD_MAX_ZEILEN-1)*LCD_MAX_SPALTEN;i++)
 {
 	LCD_PrintChar(' ');
 }
}
