
//*****************************************************************************
//
// File Name	: 'global.h'
// Title			: AVR project global include 
// Author		: Pascal Stang
// Created		: 7/12/2001
// Revised		: 4/30/2002
// Version		: 1.1
// Target MCU	: Atmel AVR series
// Editor Tabs	: 3
//
//	Description : This include file is designed to contain items useful to all
//					code files and projects.
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************


#ifndef GLOBAL_H
#define GLOBAL_H
 
// AVR-GCC compiler compatibility
// avr-gcc compiler v3.1.x and older doesn't support outb() and inb()
#ifndef outb
	// if necessary
	// convert outb and inb to outp and inp
	#define	outb(addr, data)	outp(data, addr)
	#define	inb(addr)			inp(addr)
#endif


// port address helpers
#define DDR(x) ((x)-1)    // address of data direction register of port x
#define PIN(x) ((x)-2)    // address of input register of port x

// min/max macros
#define MIN(a,b)			((a<b)?(a):(b))
#define MAX(a,b)			((a>b)?(a):(b))
#define ABS(x)				((x>0)?(x):(-x))

// macro for reading 16bit words from program memory
#define PRG_RDW(a)		( (PRG_RDB((unsigned char*)(a)) & 0x00FF) | ((PRG_RDB((unsigned char*)(a)+1))<<8) )

// constants
#define PI		3.14159265359

#ifndef WIN32
	// true/false defines
	#define FALSE	0
	#define TRUE	-1
#endif

// datatype definitions macros
typedef unsigned char  u08;
typedef          char  s08;
typedef unsigned short u16;
typedef          short s16;
typedef unsigned long  u32;
typedef          long  s32;
typedef unsigned long long u64;
typedef          long long s64;

// maximum value that can be held
// by unsigned data types (8,16,32bits)
#define MAX_U08	255
#define MAX_U16	65535
#define MAX_U32	4294967296

// maximum values that can be held
// by signed data types (8,16,32bits)
#define MIN_S08	-128
#define MAX_S08	127
#define MIN_S16	-32768
#define MAX_S16	32767
#define MIN_S32	-2147483648
#define MAX_S32	2147483647

#ifndef WIN32
	// more type redefinitions
	typedef unsigned char   BOOL;
	typedef unsigned char	BYTE;
	typedef unsigned int	WORD;
	typedef unsigned long	DWORD;

	typedef unsigned char	UCHAR;
	typedef unsigned int	UINT;
	typedef unsigned short  USHORT;
	typedef unsigned long	ULONG;

	typedef char  CHAR;
	typedef int   INT;
	typedef long  LONG;
#endif

// system dependent defines

// CPU clock speed
//#define F_CPU        1000000               		// 16MHz processor
//#define F_CPU        7372800               		// 7.37MHz processor
#define F_CPU        8000000               		// 4MHz processor
//#define F_CPU        3686400               		// 3.69MHz processor
#define CYCLES_PER_US ((F_CPU+500000)/1000000) 	// cpu cycles per microsecond

// address space defines
// internal ram
#define INT_RAM_BASE					0x0100
#define INT_RAM_END					0x10ff


#define mem(addr) 		( *(char*)(addr) )
#define memw(addr) 		( *(int*)(addr) )
#define io(addr) 		( *(char*)(addr+IO_ADDR) )
#define iow(addr) 		( *(int*)(addr+IO_ADDR) )
#define IO_BOARD_ADDR		(2)
#define IO2_BOARD_ADDR		(1)
#define IO_BOARD_ADRR		(IO_BOARD_ADDR)
#define IO_OUT			(io(IO_BOARD_ADDR)=0x00ff & ioout; io(IO_BOARD_ARRD+0x10)=ioout>>8;)
				
#define IO_BYTE(mem_addr) (*(char *)(mem_addr))
#define IO_WORD(mem_addr) (*(int *)(mem_addr))


#define setbit(value,bit)   	( value =value|BV(bit) )
#define clearbit(value,bit) 	( value &= ~BV(bit) )
#define bitset(value,bit)   	( value & (1<<(bit)) )
#define bitisset(value,bit)   	( value & (1<<(bit)) )
#define bitisclear(value,bit)   ( !(value & (1<<(bit))) )

#define TIME_IS_OVER(time, numofms) (GetTime()>(time)+(numofms))


#define IO_ADDR (0xffff-255)

#define EVER ;;

//############################


#endif  
